@@ -53,7 +53,11 @@
 #define ZIP(x)	 ZIPJAZ(x,96, 64, 32, 0)
 #define RZIP(x)	 ZIPJAZ(x,96, 64, 32, SCSI_FLAG|PRIV_FLAG)
 
+#define REMOTE    {"$DISPLAY", 'X', 0,0, 0,0, 0,0,0L, DEF_ARG0(FLOPPYD_FLAG)}
 
+
+
+#if defined(INIT_GENERIC) || defined(INIT_NOOP)
 static int compare_geom(struct device *dev, struct device *orig_dev)
 {
 	if(IS_MFORMAT_ONLY(orig_dev))
@@ -64,6 +68,7 @@ static int compare_geom(struct device *dev, struct device *orig_dev)
 	       orig_dev->heads != dev->heads ||
 	       orig_dev->sectors  != dev->sectors);
 }
+#endif
 
 #define devices const_devices
 
@@ -73,7 +78,8 @@ static int compare_geom(struct device *dev, struct device *orig_dev)
 struct device devices[] = {
    {"/dev/floppy0", 'A', GENFD },
    {"/dev/rdsk/c104d0s31", 'J', JAZ(O_EXCL) },
-   {"/dev/rdsk/c105d0s31", 'Z', ZIP(O_EXCL) }
+   {"/dev/rdsk/c105d0s31", 'Z', ZIP(O_EXCL) },
+   REMOTE
 };
 #endif /* aux */
 
@@ -82,6 +88,7 @@ struct device devices[] = {
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/fd1440.0", 	'A', MHD312 },
+	REMOTE
 };
 #endif
 
@@ -90,6 +97,7 @@ struct device devices[] = {
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/disk/floppy/raw", 	'A', MHD312 },
+	REMOTE
 };
 #endif /* BEBOX */
 
@@ -112,10 +120,11 @@ struct device devices[] = {
  	{"/dev/rfloppy/c20Ad1s0",	'B', FHD312 },
  	{"/dev/rscsi",			'C', GENHD },
 #endif
-	{"/dev/rdsk/c201d4",		'J', JAZ(O_EXCL) },
-	{"/dev/rdsk/c201d4s0",		'J', JAZ(O_EXCL) },
-	{"/dev/rdsk/c201d5",		'Z', ZIP(O_EXCL) },
-	{"/dev/rdsk/c201d5s0",		'Z', ZIP(O_EXCL) },
+	{"/dev/rdsk/c201d4",		'J', RJAZ(O_EXCL) },
+	{"/dev/rdsk/c201d4s0",		'J', RJAZ(O_EXCL) },
+	{"/dev/rdsk/c201d5",		'Z', RZIP(O_EXCL) },
+	{"/dev/rdsk/c201d5s0",		'Z', RZIP(O_EXCL) },
+	REMOTE
 };
 
 #ifdef HAVE_SYS_FLOPPY
@@ -182,6 +191,7 @@ struct device devices[] = {
 	{ "/dev/fd1",		'B', GENFD},
 #endif /* CPU_i386 */
 #endif /*mips*/
+	REMOTE
 };
 #endif
 
@@ -190,6 +200,7 @@ struct device devices[] = {
 struct device devices[] = {
 	{"/dev/rfd0a",		'A', GENFD}, /* guessed */
 	{"/dev/rfd0c",		'A', GENFD}, /* guessed */
+	REMOTE
 };
 
 #endif
@@ -207,6 +218,7 @@ struct device devices[] = {
 	{"/dev/rdsk/0p1",	'C', GENHD},
 	{"/usr/vpix/defaults/C:",'D',12, 0, 0, 0, 0,8704L,DEF_ARG0},
 	{"$HOME/vpix/C:", 	'E', 12, 0, 0, 0, 0,8704L,MDEF_ARG},
+	REMOTE
 };
 #else
 /* contributed by larry.jones@sdrc.com (Larry Jones) */
@@ -215,7 +227,8 @@ struct device devices[] = {
 	{"/dev/rfd1",		'B', GEN},
 	{"/dev/rdsk/0p1",	'C', GEN},
 	{"/usr/vpix/defaults/C:",'D', GEN, 1},
-	{"$HOME/vpix/C:", 	'E', GEN, 1}
+	{"$HOME/vpix/C:", 	'E', GEN, 1},
+	REMOTE
 };
 
 #include <sys/vtoc.h>
@@ -228,7 +241,7 @@ int ioctl(int, int, void *);
 
 static int get_parameters(int fd, struct generic_floppy_struct *floppy)
 {
-	off_t off;
+	mt_off_t off;
 	char buf[512];
 
 	off = lseek(fd, 0, SEEK_CUR);
@@ -267,6 +280,7 @@ static int set_parameters(int fd, struct generic_floppy_struct *floppy,
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/rfd0", 'A', GENFD},
+	REMOTE
 };
 #endif /* CPU_i370 */
 
@@ -274,7 +288,8 @@ struct device devices[] = {
 /* modified by Federico Bianchi */
 #define predefined_devices
 struct device devices[] = {
-  {"/dev/fd0",'A',GENFD},
+	{"/dev/fd0",'A',GENFD},
+	REMOTE
 };
 #endif /* aix */
 
@@ -283,7 +298,8 @@ struct device devices[] = {
 /* modified by Chris Samuel <chris@rivers.dra.hmg.gb> */
 #define predefined_devices
 struct device devices[] = {
-  {"/dev/fd0c",'A',GENFD},
+	{"/dev/fd0c",'A',GENFD},
+	REMOTE
 };
 #endif /* OS_osf4 */
 
@@ -338,6 +354,7 @@ struct device devices[] = {
 #endif	/* USING_VOLD */
 	{"/dev/rdsk/c0t4d0s2", 'J', RJAZ(O_NDELAY)},
 	{"/dev/rdsk/c0t5d0s2", 'Z', RZIP(O_NDELAY)},
+	REMOTE
 };
 
 
@@ -394,6 +411,7 @@ static inline int set_parameters(int fd, struct generic_floppy_struct *floppy,
 struct device devices[] = {
 	{"/dev/rfdl0c",	'A', FDD312},
 	{"/dev/rfd0c",	'A', FHD312},
+	REMOTE
 };
 #endif /* OS_sunos3 */
 
@@ -405,6 +423,7 @@ struct device devices[] = {
 	{"/dev/fd1135ds18",	'B', FHD312},
 	{"/dev/fd1135ds9",	'B', FDD312},
 	{"/dev/hd0d",		'C', GENHD},
+	REMOTE
 };
 #endif /* OS_xenix */
 
@@ -418,6 +437,7 @@ struct device devices[] = {
 	{ "/dev/fd1135ds9",	'B', FDD514},
 	{ "/dev/fd1",		'B', GENFD},
 	{ "/dev/hd0d",		'C', GENHD},
+	REMOTE
 };
 #endif /* OS_sco */
 
@@ -429,6 +449,7 @@ struct device devices[] = {
   { "/dev/rdsk/fds0d2.3.5",	'A', FDD312},
   { "/dev/rdsk/fds0d2.96",	'A', FHD514},
   {"/dev/rdsk/fds0d2.48",	'A', FDD514},
+  REMOTE
 };
 #endif /* OS_irix */
 
@@ -442,6 +463,7 @@ struct device devices[] = {
 	{"/dev/rfd0c",	'A', GENFD},
 	{"/dev/rsd4c",	'J', RJAZ(O_NDELAY)},
 	{"/dev/rsd5c",	'Z', RZIP(O_NDELAY)},
+	REMOTE
 };
 
 /*
@@ -514,6 +536,7 @@ struct device devices[] = {
 	{"/dev/flbm60", 'C', MDDsmall},
 	{"/dev/flbm60", 'D', MSS},
 	{"/dev/flbm60", 'E', MSSsmall},
+	REMOTE
 };
 #endif /* DPX1000 */
 
@@ -526,6 +549,7 @@ struct device devices[] = {
 	{"/dev/easyfb", 'C', MDDsmall},
 	{"/dev/easyfb", 'D', MSS},
 	{"/dev/easyfb", 'E', MSSsmall},
+	REMOTE
 };
 #endif /* OS_bosx */
 
@@ -630,14 +654,30 @@ int analyze_one_reply(RawRequest_t *raw_cmd, int *bytes, int do_print)
 {
 	
 	if(raw_cmd->reply_count == 7) {
-		*bytes = (raw_cmd->reply[5] - raw_cmd->cmd[4]);
+		int end;
+		
+		if (raw_cmd->reply[3] != raw_cmd->cmd[2]) {
+			/* end of cylinder */
+			end = raw_cmd->cmd[6] + 1;
+		} else {
+			end = raw_cmd->reply[5];
+		}
+
+		*bytes = end - raw_cmd->cmd[4];
 		/* FIXME: over/under run */
-		*bytes = *bytes << (7 + raw_cmd->cmd[6]);
+		*bytes = *bytes << (7 + raw_cmd->cmd[5]);
 	} else
 		*bytes = 0;       
 
 	switch(raw_cmd->reply[0] & 0xc0){
 		case 0x40:
+			if ((raw_cmd->reply[0] & 0x38) == 0 &&
+			    (raw_cmd->reply[1]) == 0x80 &&
+			    (raw_cmd->reply[2]) == 0) {
+				*bytes += 1 << (7 + raw_cmd->cmd[5]);
+				break;
+			}
+
 			if ( raw_cmd->reply[1] & ST1_WP ){
 				*bytes = 0;
 				fprintf(stderr,
@@ -658,12 +698,13 @@ int analyze_one_reply(RawRequest_t *raw_cmd, int *bytes, int do_print)
 				"abnormal termination caused by polling\n");
 			return -1;
 		default:
-#ifdef FD_RAW_MORE
-			if(raw_cmd->flags & FD_RAW_MORE)
-				return 1;
-#endif
-			return 0;
+			break;
 	}	
+#ifdef FD_RAW_MORE
+	if(raw_cmd->flags & FD_RAW_MORE)
+		return 1;
+#endif
+	return 0;
 }
 
 #define predefined_devices
@@ -674,6 +715,7 @@ struct device devices[] = {
 	{"/dev/sdb4",'J', GENHD },
 	{"/dev/sdb4",'Z', GENHD },
 	/*	{"/dev/sda4",'D', GENHD },*/
+	REMOTE
 };
 
 /*
@@ -751,6 +793,7 @@ struct device devices[] = {
 	{"/dev/rfd0.720",  'A', FDD312},
 	{"/dev/rfd1.1200", 'B', MHD514},
 	{"/dev/sd0s1",     'C', GENHD},
+	REMOTE
 };
 #endif /* __FreeBSD__ */
  
@@ -765,6 +808,7 @@ struct device devices[] = {
 	{"/dev/rfd1d", 'B', FDD514},
 	{"/dev/rfd1d", 'T', MDD514},
 	{"/dev/rwd0d", 'C', 16, 0, 0, 0, 0, 0, 63L*512L, DEF_ARG0(0)},
+	REMOTE
 };
 #endif /* OS_NetBSD */
 
@@ -776,6 +820,7 @@ struct device devices[] = {
 struct device devices[] = {
 	{"/dev/rfp020",		'A', 12,O_NDELAY,40,2, 9, 0, MDEF_ARG},
 	{"/usr/bin/DOS/dvd000", 'C', GENFD},
+	REMOTE
 };
 
 #undef INIT_NOOP
@@ -820,6 +865,7 @@ int init_geom(int fd, struct device *dev, struct device *orig_dev,
 #define predefined_devices
 struct device devices[] = {
 	{"/dev/rfd0c",		'A', GENFD},
+	REMOTE
 };
 #endif
 #endif
@@ -829,20 +875,23 @@ struct device devices[] = {
 #ifndef predefined_devices
 #define predefined_devices
 struct device devices[] = {
-  {"/dev/fd0a", 'A',  MHD312 } };
+	{"/dev/fd0a", 'A',  MHD312 } };
+	REMOTE
 #endif
 #endif
 
 
 #ifdef OS_nextstep
 #define predefined_devices
+struct device devices[] = {
 #ifdef CPU_m68k
-struct device devices[] = {
-  {"/dev/rfd0b", 'A', MED312 } };
+	{"/dev/rfd0b", 'A', MED312 },
+	REMOTE
 #else
-struct device devices[] = {
-  {"/dev/rfd0b", 'A', MHD312 } };
+	{"/dev/rfd0b", 'A', MHD312 },
+	REMOTE
 #endif
+};
 #endif
 
 
@@ -852,6 +901,7 @@ struct device devices[] = {
 struct device devices[] = {
       {"/dev/fpd0",   'A', FHD312},
       {"/dev/fpd0",   'A', FDD312},
+	  REMOTE
 };
 #else
 #define predefined_devices
@@ -862,6 +912,7 @@ struct device devices[] = {
 	{"/dev/rdsk/f03ht",	'A', FHD312},
 	{"/dev/rdsk/f03dt",	'A', FDD312},
 	{"/dev/rdsk/dos",	'C', GENHD},
+	REMOTE
 };
 #endif
 #endif /* sysv4 */
