@@ -5,8 +5,12 @@
 #ifdef HAVE_SYS_SYSMACROS_H
 
 #include <sys/sysmacros.h>
+#ifndef MAJOR
 #define MAJOR(dev) major(dev)
+#endif  /* MAJOR not defined */
+#ifndef MINOR
 #define MINOR(dev) minor(dev)
+#endif  /* MINOR not defined */
 
 #else
  
@@ -75,12 +79,14 @@ UNUSED(static inline void RR_SETSECTOR(struct floppy_raw_cmd *request,
 				       int sector))
 {
 	request->cmd[4] = sector;
+	request->cmd[6] = sector-1;
 }
 
 UNUSED(static inline void RR_SETSIZECODE(struct floppy_raw_cmd *request, 
 					 int sizecode))
 {
 	request->cmd[5] = sizecode;
+	request->cmd[6]++;
 	request->length += 128 << sizecode;
 }
 
@@ -94,13 +100,12 @@ static inline void RR_SETEND(struct floppy_raw_cmd *request, int end)
 UNUSED(static inline void RR_SETDIRECTION(struct floppy_raw_cmd *request, 
 					  int direction))
 {
-
 	if(direction == MT_READ) {
 		request->flags |= FD_RAW_READ;
-		request->cmd[0] = FD_READ;
+		request->cmd[0] = FD_READ & ~0x80;
 	} else {
 		request->flags |= FD_RAW_WRITE;
-		request->cmd[0] = FD_WRITE;
+		request->cmd[0] = FD_WRITE & ~0x80;
 	}
 }
 
