@@ -20,11 +20,13 @@ typedef struct Filter_t {
 
 /* read filter filters out messy dos' bizarre end of lines and final 0x1a's */
 
-static int read_filter(Stream_t *Stream, char *buf, off_t where, size_t len)
+static int read_filter(Stream_t *Stream, char *buf, mt_off_t iwhere, size_t len)
 {
 	DeclareThis(Filter_t);
 	int i,j,ret;
 
+	off_t where = truncBytes32(iwhere);
+
 	if ( where != This->unixpos ){
 		fprintf(stderr,"Bad offset\n");
 		exit(1);
@@ -35,7 +37,7 @@ static int read_filter(Stream_t *Stream, char *buf, off_t where, size_t len)
 	}
 	This->rw = F_READ;
 	
-	ret = READS(This->Next, buf, This->dospos, len);
+	ret = READS(This->Next, buf, (mt_off_t) This->dospos, len);
 	if ( ret < 0 )
 		return ret;
 
@@ -53,12 +55,15 @@ static int read_filter(Stream_t *Stream, char *buf, off_t where, size_t len)
 	return j;
 }
 
-static int write_filter(Stream_t *Stream, char *buf, off_t where, size_t len)
+static int write_filter(Stream_t *Stream, char *buf, mt_off_t iwhere, 
+						size_t len)
 {
 	DeclareThis(Filter_t);
 	int i,j,ret;
 	char buffer[1025];
 
+	off_t where = truncBytes32(iwhere);
+
 	if(This->unixpos == -1)
 		return -1;
 
@@ -85,7 +90,7 @@ static int write_filter(Stream_t *Stream, char *buf, off_t where, size_t len)
 	}
 	This->unixpos += j;
 
-	ret = force_write(This->Next, buffer, This->dospos, i);
+	ret = force_write(This->Next, buffer, (mt_off_t) This->dospos, i);
 	if(ret >0 )
 		This->dospos += ret;
 	if ( ret != i ){
@@ -103,7 +108,7 @@ static int free_filter(Stream_t *Stream)
 
 	/* write end of file */
 	if (This->rw == F_WRITE)
-		return force_write(This->Next, &buffer, This->dospos, 1);
+		return force_write(This->Next, &buffer, (mt_off_t) This->dospos, 1);
 	else
 		return 0;
 }
