@@ -10,6 +10,7 @@
 #include "mtools.h"
 #include "fsP.h"
 #include "plain_io.h"
+#include "floppyd_io.h"
 #include "xdf_io.h"
 #include "buffer.h"
 
@@ -64,7 +65,7 @@ static int get_media_type(Stream_t *St, struct bootsector *boot)
 		char temp[512];
 		/* old DOS disk. Media descriptor in the first FAT byte */
 		/* old DOS disk always have 512-byte sectors */
-		if (force_read(St,temp,512,512) == 512)
+		if (force_read(St,temp,(mt_off_t) 512,512) == 512)
 			media = (unsigned char) temp[0];
 		else
 			media = 0;
@@ -83,7 +84,7 @@ Stream_t *GetFs(Stream_t *Fs)
 
 Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		      struct bootsector *boot,
-		      char *name, int *media)
+		      char *name, int *media, mt_size_t *maxSize)
 {
 	char errmsg[80];
 	Stream_t *Stream;
@@ -103,18 +104,26 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		strcpy(name, getVoldName(dev, name));
 #endif
 
-
+		Stream = 0;
 #ifdef USE_XDF
 		Stream = XdfOpen(out_dev, name, mode, errmsg, 0);
-#else
-		Stream = 0;
+		if(Stream) {
+			out_dev->use_2m = 0x7f;
+			*maxSize = max_off_t_31;
+		}
+#endif
+
+#ifdef USE_FLOPPYD
+		if(!Stream) {
+			Stream = FloppydOpen(out_dev, dev, name, mode, errmsg, 0, 1);
+			if(Stream)
+				*maxSize = max_off_t_31;
+		}
 #endif
 
 		if (!Stream)
 			Stream = SimpleFileOpen(out_dev, dev, name, mode,
-						errmsg, 0, 1);
-		else
-			out_dev->use_2m = 0x7f;
+									errmsg, 0, 1, maxSize);
 
 		if( !Stream)
 			continue;
@@ -173,6 +182,7 @@ Stream_t *fs_init(char drive, int mode)
 	char name[EXPAND_BUF];
 	int cylinder_size;
 	struct device dev;
+	mt_size_t maxSize;
 
 	struct bootsector boot0;
 #define boot (&boot0)
@@ -188,10 +198,14 @@ Stream_t *fs_init(char drive, int mode)
 	This->Buffer = 0;
 	This->Class = &FsClass;
 	This->preallocatedClusters = 0;
+	This->lastFatSectorNr = 0;
+	This->lastFatAccessMode = 0;
+	This->lastFatSectorData = 0;
 	This->drive = drive;
 	This->last = 0;
 
-	This->Direct = find_device(drive, mode, &dev, &boot0, name, &media);
+	This->Direct = find_device(drive, mode, &dev, &boot0, name, &media, 
+							   &maxSize);
 	if(!This->Direct)
 		return NULL;
 	
@@ -200,11 +214,10 @@ Stream_t *fs_init(char drive, int mode)
 		fprintf(stderr,"init %c: sector size too big\n", drive);
 		return NULL;
 	}
-	for(i=0; i<16; i++) {
-		if(1 << i == This->sector_size)
-			break;
-	}
-	if(i == 16) {
+
+	i = log_2(This->sector_size);
+
+	if(i == 24) {
 		fprintf(stderr, 
 			"init %c: sector size (%d) not a small power of two\n",
 			drive, This->sector_size);
@@ -253,10 +266,18 @@ Stream_t *fs_init(char drive, int mode)
 		This->num_fat = boot0.nfat;
 	}
 
+	if (tot_sectors >= (maxSize >> This->sectorShift)) {
+		fprintf(stderr, "Big disks not supported on this architecture\n");
+		exit(1);
+	}
+
 	if(!mtools_skip_check && (tot_sectors % dev.sectors)){
 		fprintf(stderr,
 			"Total number of sectors not a multiple of"
 			" sectors per track!\n");
+		fprintf(stderr,
+			"Add mtools_skip_check=1 to your .mtoolsrc file "
+			"to skip this test\n");
 		exit(1);
 	}
 
@@ -325,9 +346,9 @@ char getDrive(Stream_t *Stream)
 		return This->drive;
 }
 
-int fsPreallocate(Fs_t *Fs, int size)
+int fsPreallocateClusters(Fs_t *Fs, long size)
 {
-	if(size > 0 && getfreeMin((Stream_t *)Fs, size) != 1)
+	if(size > 0 && getfreeMinClusters((Stream_t *)Fs, size) != 1)
 		return -1;
 
 	Fs->preallocatedClusters += size;
