@@ -35,7 +35,7 @@ int unix_loop(Stream_t *Stream, MainParam_t *mp, char *arg, int follow_dir_link)
 	/*	mp->dir.attr = ATTR_ARCHIVE;*/
 	mp->loop = _unix_loop;
 	if((mp->lookupflags & DO_OPEN)){
-		mp->File = SimpleFileOpen(0, 0, arg, O_RDONLY, 0, 0, 0);
+		mp->File = SimpleFileOpen(0, 0, arg, O_RDONLY, 0, 0, 0, 0);
 		if(!mp->File){
 			perror(arg);
 #if 0
@@ -534,6 +534,13 @@ char *mpBuildUnixFilename(MainParam_t *mp)
 		return 0;
 	strcpy(ret, mp->unixTarget);
 	if(*target) {
+#if 1 /* fix for 'mcopy -n x:file existingfile' -- H. Lermen 980816 */
+		if(!mp->targetName && !mp->targetDir) {
+			struct stat buf;
+			if (!stat(ret, &buf) && !S_ISDIR(buf.st_mode))
+				return ret;
+		}
+#endif
 		strcat(ret, "/");
 		strcat(ret, target);
 	}
