@@ -22,11 +22,45 @@ static int attrib_file(direntry_t *entry, MainParam_t *mp)
 {
 	Arg_t *arg=(Arg_t *) mp->arg;
 
-	entry->dir.attr = (entry->dir.attr & arg->remove) | arg->add;
-	dir_write(entry);
+	if(entry->entry != -3) {
+		/* if not root directory, change it */
+		entry->dir.attr = (entry->dir.attr & arg->remove) | arg->add;
+		dir_write(entry);
+	}
 	return GOT_ONE;
 }
 
+static int replay_attrib(direntry_t *entry, MainParam_t *mp)
+{
+	if ( (IS_ARCHIVE(entry) && IS_DIR(entry)) ||
+		 (!IS_ARCHIVE(entry) && !IS_DIR(entry)) ||
+		 IS_SYSTEM(entry) || IS_HIDDEN(entry)) {
+
+		printf("mattrib ");
+
+		if (IS_ARCHIVE(entry) && IS_DIR(entry)) {
+			printf("+a ");
+		}
+
+		if (!IS_ARCHIVE(entry) && !IS_DIR(entry)) {
+			printf("-a ");
+		}
+
+		if (IS_SYSTEM(entry)) {
+			printf("+s ");
+		}
+
+		if (IS_HIDDEN(entry)) {
+			printf("+h ");
+		}
+
+		fprintPwd(stdout, entry);
+		printf("\n");
+	}
+	return GOT_ONE;
+}
+
+
 
 static int view_attrib(direntry_t *entry, MainParam_t *mp)
 {
@@ -118,6 +152,7 @@ void mattrib(int argc, char **argv, int type)
 	int view;
 	int c;
 	int concise;
+	int replay;
 	char *ptr;
 
 	arg.add = 0;
@@ -126,12 +161,16 @@ void mattrib(int argc, char **argv, int type)
 	arg.doPrintName = 1;
 	view = 0;
 	concise = 0;
-
-	while ((c = getopt(argc, argv, "/ahrsAHRSX")) != EOF) {
+	replay = 0;
+	
+	while ((c = getopt(argc, argv, "/ahrsAHRSXp")) != EOF) {
 		switch (c) {
 			default:
 				arg.remove &= ~letterToCode(c);
 				break;
+			case 'p':
+				replay = 1;
+				break;
 			case '/':
 				arg.recursive = 1;
 				break;
@@ -170,6 +209,8 @@ void mattrib(int argc, char **argv, int type)
 			arg.doPrintName = (argc - optind > 1 ||
 					   arg.recursive ||
 					   strpbrk(argv[optind], "*[?") != 0);
+		} else if (replay) {
+			arg.mp.callback = replay_attrib;
 		} else
 			arg.mp.callback = view_attrib;
 		arg.mp.openflags = O_RDONLY;
