@@ -22,7 +22,7 @@
  * Preserve the file modification times after the fclose()
  */
 
-static void set_mtime(const char *target, long mtime)
+static void set_mtime(const char *target, time_t mtime)
 {
 	if (target && strcmp(target, "-") && mtime != 0L) {
 #ifdef HAVE_UTIMES
@@ -64,7 +64,7 @@ typedef struct Arg_t {
 static int unix_write(direntry_t *entry, MainParam_t *mp, int needfilter)
 {
 	Arg_t *arg=(Arg_t *) mp->arg;
-	long mtime;
+	time_t mtime;
 	Stream_t *File=mp->File;
 	Stream_t *Target, *Source;
 	struct stat stbuf;
@@ -119,7 +119,7 @@ static int unix_write(direntry_t *entry, MainParam_t *mp, int needfilter)
 
 	if ((Target = SimpleFileOpen(0, 0, unixFile,
 				     O_WRONLY | O_CREAT | O_TRUNC,
-				     errmsg, 0, 0))) {
+				     errmsg, 0, 0, 0))) {
 		ret = 0;
 		if(needfilter && arg->textmode){
 			Source = open_filter(COPY(File));
@@ -171,7 +171,7 @@ static int makeUnixDir(char *filename)
 static int unix_copydir(direntry_t *entry, MainParam_t *mp)
 {
 	Arg_t *arg=(Arg_t *) mp->arg;
-	long mtime;
+	time_t mtime;
 	Stream_t *File=mp->File;
 	int ret;
 	char *unixFile;
@@ -246,14 +246,14 @@ static int writeit(char *dosname,
 	Stream_t *Target;
 	time_t now;
 	int type, fat, ret;
-	long date;
-	size_t filesize, newsize;
+	time_t date;
+	mt_size_t filesize, newsize;
 	Arg_t *arg = (Arg_t *) arg0;
 
 
 
 	if (arg->mp.File->Class->get_data(arg->mp.File,
-					  & date, &filesize, &type, 0) < 0 ){
+									  & date, &filesize, &type, 0) < 0 ){
 		fprintf(stderr, "Can't stat source file\n");
 		return -1;
 	}
@@ -271,7 +271,7 @@ static int writeit(char *dosname,
 		return -1;
 
 	/* will it fit? */
-	if (!getfreeMin(arg->mp.targetDir, filesize))
+	if (!getfreeMinBytes(arg->mp.targetDir, filesize))
 		return -1;
 	
 	/* preserve mod time? */
@@ -299,7 +299,8 @@ static int writeit(char *dosname,
 		fat_free(arg->mp.targetDir, fat);
 		return -1;
 	} else {
-		mk_entry(dosname, arg->attr, fat, newsize, now, &entry->dir);
+		mk_entry(dosname, arg->attr, fat, truncBytes32(newsize),
+				 now, &entry->dir);
 		return 0;
 	}
 }
@@ -351,7 +352,7 @@ static int dos_copydir(direntry_t *entry, MainParam_t *mp)
 	Arg_t * arg = (Arg_t *) (mp->arg);
 	Arg_t newArg;
 	time_t now;
-	long date;
+	time_t date;
 	int ret;
 	const char *targetName = mpPickTargetName(mp);
 
