@@ -29,8 +29,8 @@ static int dirsOnDrive; /* number of listed directories on this drive */
 
 static int debug = 0; /* debug mode */
 
-static int bytesInDir;
-static int bytesOnDrive;
+static mt_size_t bytesInDir;
+static mt_size_t bytesOnDrive;
 static Stream_t *RootDir;	
 
 
@@ -97,12 +97,20 @@ static inline void print_time(struct directory *dir)
 /*
  * Return a number in dotted notation
  */
-static const char *dotted_num(unsigned long num, int width, char **buf)
+static const char *dotted_num(mt_size_t num, int width, char **buf)
 {
 	int      len;
 	register char *srcp, *dstp;
 	int size;
 
+	unsigned long numlo;
+	unsigned long numhi;
+
+	if (num < 0) {
+	    /* warn about negative numbers here.  They should not occur */
+	    fprintf(stderr, "Invalid negative number\n");
+	}
+
 	size = width + width;
 	*buf = malloc(size+1);
 
@@ -112,7 +120,15 @@ static const char *dotted_num(unsigned long num, int width, char **buf)
 	/* Create the number in maximum width; make sure that the string
 	 * length is not exceeded (in %6ld, the result can be longer than 6!)
 	 */
-	sprintf(*buf, "%.*ld", size, num);
+
+	numlo = num % 1000000000;
+	numhi = num / 1000000000;
+
+	if(numhi && size > 9) {
+		sprintf(*buf, "%.*lu%09lu", size-9, numhi, numlo);
+	} else {
+		sprintf(*buf, "%.*lu", size, numlo);
+	}
 
 	for (srcp=*buf; srcp[1] != '\0'; ++srcp)
 		if (srcp[0] == '0')
@@ -176,7 +192,7 @@ static inline void print_volume_label(Stream_t *Dir, char drive)
 }
 
 
-static void printSummary(int files, int bytes)
+static void printSummary(int files, mt_size_t bytes)
 {
 	if(!filesInDir)
 		printf("No files\n");
@@ -188,7 +204,7 @@ static void printSummary(int files, int bytes)
 		else
 			putchar('s');
 		printf("       %s bytes\n",
-		       dotted_num(bytes,13, &s1));
+		       dotted_num(bytes, 13, &s1));
 		if(s1)
 			free(s1);
 	}
@@ -209,9 +225,9 @@ static void leaveDrive(void)
 			printSummary(filesOnDrive, bytesOnDrive);
 		}
 		if(RootDir && !fast) {
-			int blocks = getfree(RootDir);
-			printf("                      %s bytes free\n\n",
-			       dotted_num(blocks,13, &s1));
+			mt_off_t bytes = getfree(RootDir);
+			printf("                  %s bytes free\n\n",
+			       dotted_num(bytes,17, &s1));
 		}
 		if(s1)
 			free(s1);
@@ -285,7 +301,7 @@ static void enterDirectory(Stream_t *Dir)
 
 static int list_file(direntry_t *entry, MainParam_t *mp)
 {
-	size_t size;
+	unsigned long size;
 	int i;
 	int Case;
 
@@ -362,8 +378,8 @@ static int list_file(direntry_t *entry, MainParam_t *mp)
 	filesOnDrive++;
 	filesInDir++;
 
-	bytesOnDrive += size;
-	bytesInDir += size;
+	bytesOnDrive += (mt_size_t) size;
+	bytesInDir += (mt_size_t) size;
 	return GOT_ONE;
 }
 
@@ -444,7 +460,7 @@ void mdir(int argc, char **argv, int type)
 				break;
 			case 'X':
 				concise = 1;
-				recursive = 1;
+				/*recursive = 1;*/
 				break;
 			case '/':
 				recursive = 1;
