@@ -2,6 +2,8 @@
  * mformat.c
  */
 
+#define DONT_NEED_WAIT
+
 #include "sysincludes.h"
 #include "msdos.h"
 #include "mtools.h"
@@ -9,6 +11,7 @@
 #include "fsP.h"
 #include "file.h"
 #include "plain_io.h"
+#include "floppyd_io.h"
 #include "nameclash.h"
 #include "buffer.h"
 #ifdef USE_XDF
@@ -20,6 +23,12 @@
 #define abs(x) ((x)>0?(x):-(x))
 #endif
 
+#ifdef OS_linux
+#include "linux/hdreg.h"
+#include "linux/fs.h"
+#endif
+
+
 extern int errno;
 
 static void init_geometry_boot(struct bootsector *boot, struct device *dev,
@@ -107,7 +116,7 @@ static void init_geometry_boot(struct bootsector *boot, struct device *dev,
 			boot->jump[0] = 0xeb;
 			boot->jump[1] = 0;
 			boot->jump[2] = 0x90;
-			strncpy(boot->banner, "MTOOL391", 8);
+			strncpy(boot->banner, "MTOOL396", 8);
 			/* It looks like some versions of DOS are
 			 * rather picky about this, and assume default
 			 * parameters without this, ignoring any
@@ -150,7 +159,7 @@ static int comp_fat_bits(Fs_t *Fs, int estimate,
 	}
 
 	if(needed_fat_bits == 32 && !fat32 && abs(estimate) !=32){
-		fprintf(stderr,"Warning: Using 32 bit FAT.  Drive will only be accessibly by Win95 OEM\n");
+		fprintf(stderr,"Warning: Using 32 bit FAT.  Drive will only be accessibly by Win95 OEM / Win98\n");
 	}
 
 	if(!estimate) {
@@ -243,7 +252,8 @@ static inline void format_root(Fs_t *Fs, char *label, struct bootsector *boot)
 	} else
 		dirlen = Fs->dir_len; 
 	for (i = 0; i < dirlen; i++)
-		WRITES(RootDir, buf, i * Fs->sector_size,  Fs->sector_size);
+		WRITES(RootDir, buf, sectorsToBytes((Stream_t*)Fs, i),  
+			   Fs->sector_size);
 
 	ch.ignore_entry = 1;
 	if(label[0])
@@ -564,6 +574,8 @@ void mformat(int argc, char **argv, int dummy)
  	int serial_set;
 	int fsVersion;
 
+	mt_off_t maxSize;
+
 	int Atari = 0; /* should we add an Atari-style serial number ? */
  
 	hs = hs_set = 0;
@@ -708,13 +720,23 @@ void mformat(int argc, char **argv, int dummy)
 #endif
 
 #ifdef USE_XDF
-		if(!format_xdf)
+		if(!format_xdf) {
 #endif
-			Fs.Direct = SimpleFileOpen(&used_dev, dev, name,
-						   O_RDWR | create,
-						   errmsg, 0, 1);
+			Fs.Direct = 0;
+#ifdef USE_FLOPPYD
+			Fs.Direct = FloppydOpen(&used_dev, dev, name, O_RDWR | create,
+									errmsg, 0, 1);
+			if(Fs.Direct) {
+				maxSize = max_off_t_31;
+			}
+#endif
+			if(!Fs.Direct) {			
+				Fs.Direct = SimpleFileOpen(&used_dev, dev, name,
+										   O_RDWR | create,
+										   errmsg, 0, 1, &maxSize);
+			}
 #ifdef USE_XDF
-		else {
+		} else {
 			used_dev.misc_flags |= USE_XDF_FLAG;
 			Fs.Direct = XdfOpen(&used_dev, name, O_RDWR,
 					    errmsg, &info);
@@ -728,12 +750,49 @@ void mformat(int argc, char **argv, int dummy)
 		if (!Fs.Direct)
 			continue;
 
-		/* non removable media */
+#ifdef OS_linux
+		if ((!used_dev.tracks || !used_dev.heads || !used_dev.sectors) &&
+			(!IS_SCSI(dev))) {
+			int fd= get_fd(Fs.Direct);
+			struct stat buf;
+
+			if (fstat(fd, &buf) < 0) {
+				sprintf(errmsg, "Could not stat file (%s)", strerror(errno));
+				continue;						
+			}
+
+			if (S_ISBLK(buf.st_mode)) {
+				struct hd_geometry geom;
+				long size;
+				int sect_per_track;
+
+				if (ioctl(fd, HDIO_GETGEO, &geom) < 0) {
+					sprintf(errmsg, "Could not get geometry of device (%s)",
+							strerror(errno));
+					continue;
+				}
+
+				if (ioctl(fd, BLKGETSIZE, &size) < 0) {
+					sprintf(errmsg, "Could not get size of device (%s)",
+							strerror(errno));
+					continue;
+				}
+
+				sect_per_track = geom.heads * geom.sectors;
+				used_dev.heads = geom.heads;
+				used_dev.sectors = geom.sectors;
+				used_dev.hidden = geom.start % sect_per_track;
+				used_dev.tracks = (size + used_dev.hidden) / sect_per_track;
+			}
+		}
+#endif
+
+		/* no way to find out geometry */
 		if (!used_dev.tracks || !used_dev.heads || !used_dev.sectors){
 			sprintf(errmsg, 
-				"Non-removable media is not supported "
+				"Unknown geometry "
 				"(You must tell the complete geometry "
-				"of the disk, either in /etc/mtools or "
+				"of the disk, \neither in /etc/mtools.conf or "
 				"on the command line) ");
 			continue;
 		}
@@ -747,15 +806,21 @@ void mformat(int argc, char **argv, int dummy)
 		}
 #endif
 		Fs.sector_size = 512;
-		Fs.sectorShift = 9;
-		Fs.sectorMask = 511;
 		if( !(used_dev.use_2m & 0x7f)) {
 			Fs.sector_size = 128 << (used_dev.ssize & 0x7f);
-			Fs.sectorShift  = 7 + (used_dev.ssize & 0x7f);
-			Fs.sectorMask  = Fs.sector_size - 1;
 		}
-		SET_INT(Fs.sector_size, msize);
 
+		SET_INT(Fs.sector_size, msize);
+		{
+		    int i;
+		    for(i = 0; i < 31; i++) {
+			if (Fs.sector_size == 1 << i) {
+			    Fs.sectorShift = i;
+			    break;
+			}
+		    }
+		    Fs.sectorMask = Fs.sector_size - 1;
+		}
 
 		if(!used_dev.blocksize || used_dev.blocksize < Fs.sector_size)
 			blocksize = Fs.sector_size;
@@ -857,14 +922,17 @@ void mformat(int argc, char **argv, int dummy)
 		Fs.infoSectorLoc = 0;
 		Fs.fat_start = 1;
 		calc_fs_parameters(&used_dev, tot_sectors, &Fs, boot);
-		boot->ext.old.physdrive = 0x00;
+		if (!keepBoot)
+			/* only zero out physdrive if we don't have a template
+			 * bootsector */
+			boot->ext.old.physdrive = 0x00;
 		boot->ext.old.reserved = 0;
 		boot->ext.old.dos4 = 0x29;
 		Fs.dir_start = Fs.num_fat * Fs.fat_len + Fs.fat_start;
 		Fs.clus_start = Fs.dir_start + Fs.dir_len;
 
 		if (!serial_set || Atari)
-			srandom(time (0));
+			srandom((long)time (0));
 		if (!serial_set)
 			serial=random();
 		set_dword(boot->ext.old.serial, serial);	
@@ -873,7 +941,8 @@ void mformat(int argc, char **argv, int dummy)
 		else
 			label_name(label, 0, &mangled, shortlabel);
 		strncpy(boot->ext.old.label, shortlabel, 11);
-		sprintf(boot->ext.old.fat_type, "FAT%2.2d   ", Fs.fat_bits);
+		sprintf(boot->ext.old.fat_type, "FAT%2.2d  ", Fs.fat_bits);
+		boot->ext.old.fat_type[7] = ' ';
 	}
 
 	set_word(boot->secsiz, Fs.sector_size);
@@ -891,7 +960,7 @@ void mformat(int argc, char **argv, int dummy)
 
 	if (create) {
 		WRITES(Fs.Direct, (char *) buf,
-		       Fs.sector_size * (tot_sectors-1),
+		       sectorsToBytes((Stream_t*)&Fs, tot_sectors-1),
 		       Fs.sector_size);
 	}
 
@@ -914,10 +983,10 @@ void mformat(int argc, char **argv, int dummy)
 #endif
 
 	format_root(&Fs, label, boot);
-	WRITES((Stream_t *)&Fs, (char *) boot, 0, Fs.sector_size);
+	WRITES((Stream_t *)&Fs, (char *) boot, (mt_off_t) 0, Fs.sector_size);
 	if(Fs.fat_bits == 32 && WORD(ext.fat32.backupBoot) != MAX32) {
 		WRITES((Stream_t *)&Fs, (char *) boot, 
-		       WORD(ext.fat32.backupBoot) * Fs.sector_size,
+		       sectorsToBytes((Stream_t*)&Fs, WORD(ext.fat32.backupBoot)),
 		       Fs.sector_size);
 	}
 	FLUSH((Stream_t *)&Fs); /* flushes Fs. 
