@@ -30,8 +30,8 @@ static void displayInfosector(Stream_t *Stream, struct bootsector *boot)
 
 	infosec = (InfoSector_t *) safe_malloc(WORD(secsiz));
 	force_read(Stream, (char *) infosec, 
-		   WORD(secsiz) * WORD(ext.fat32.infoSector),
-		   WORD(secsiz));
+			   (mt_off_t) WORD(secsiz) * WORD(ext.fat32.infoSector),
+			   WORD(secsiz));
 	printf("\nInfosector:\n");
 	printf("signature=0x%08x\n", _DWORD(infosec->signature));
 	if(_DWORD(infosec->count) != MAX32)
@@ -73,7 +73,7 @@ void minfo(int argc, char **argv, int type)
 		drive = toupper(argv[optind][0]);
 
 		if(! (Stream = find_device(drive, O_RDONLY, &dev, boot, 
-					   name, &media)))
+								   name, &media, 0)))
 			exit(1);
 
 		tot_sectors = DWORD(bigsect);
@@ -152,7 +152,7 @@ void minfo(int argc, char **argv, int type)
 				exit(1);
 			}
 
-			size = READS(Stream, buf, 0, size);
+			size = READS(Stream, buf, (mt_off_t) 0, size);
 			if(size < 0) {
 				perror("read boot sector");
 				exit(1);
