@@ -17,7 +17,7 @@ void printOom(void)
 char *get_homedir(void)
 {
 	struct passwd *pw;
-	int uid;
+	uid_t uid;
 	char *homedir;
 	char *username;
 	
