@@ -112,8 +112,8 @@ static inline clash_action ask_namematch(char *name, int isprimary,
 		} else {
 			fgets(ans, 9, opentty(0));
 		}
-		perm = isupper(ans[0]);
-		switch(tolower(ans[0])) {
+		perm = isupper((unsigned char)ans[0]);
+		switch(tolower((unsigned char)ans[0])) {
 			case 'a':
 				a = NAMEMATCH_AUTORENAME;
 				break;
@@ -301,9 +301,10 @@ static inline clash_action get_slots(Stream_t *Dir,
 		reason = EXISTS;
 		clear_scan(longname, ch->use_longname, ssp);
 		switch (lookupForInsert(Dir, dosname, longname, ssp,
-					ch->ignore_entry, 
-					pessimisticShortRename && 
-					ch->use_longname)) {
+								ch->ignore_entry, 
+								ch->source_entry,
+								pessimisticShortRename && 
+								ch->use_longname)) {
 			case -1:
 				return NAMEMATCH_ERROR;
 				
@@ -354,8 +355,7 @@ static inline clash_action get_slots(Stream_t *Dir,
 			entry.entry = match;
 			dir_read(&entry);
 			/* if we can't overwrite, don't propose it */
-			no_overwrite = (match == ch->source || 
-					(entry.dir.attr & 0x10));
+			no_overwrite = (match == ch->source || IS_DIR(&entry));
 		}
 	}
 	ret = process_namematch(isprimary ? longname : dosname, longname,
@@ -579,6 +579,7 @@ int mwrite_one(Stream_t *Dir,
 void init_clash_handling(ClashHandling_t *ch)
 {
 	ch->ignore_entry = -1;
+	ch->source_entry = -2;
 	ch->nowarn = 0;	/*Don't ask, just do default action if name collision */
 	ch->namematch_default[0] = NAMEMATCH_AUTORENAME;
 	ch->namematch_default[1] = NAMEMATCH_NONE;
