@@ -27,11 +27,11 @@ char *label_name(char *filename, int verbose,
 	strncpy(ans, filename, len);
 	have_lower = have_upper = 0;
 	for(i=0; i<11; i++){
-		if(islower(ans[i]))
+		if(islower((unsigned char)ans[i]))
 			have_lower = 1;
 		if(isupper(ans[i]))
 			have_upper = 1;
-		ans[i] = toupper(ans[i]);
+		ans[i] = toupper((unsigned char)ans[i]);
 
 		if(strchr("^+=/[]:,?*\\<>|\".", ans[i])){
 			*mangled = 1;
