@@ -56,8 +56,8 @@ int makeit(char *dosname,
 	int fat;
 	direntry_t subEntry;	
 
-	/* will it fit? At least one sector must be free */
-	if (!getfreeMin(targetEntry->Dir, 512))
+	/* will it fit? At least one cluster must be free */
+	if (!getfreeMinClusters(targetEntry->Dir, 1))
 		return -1;
 	
 	mk_entry(dosname, ATTR_DIR, 1, 0, arg->mtime, &targetEntry->dir);
@@ -76,6 +76,7 @@ int makeit(char *dosname,
 	mk_entry("..         ", ATTR_DIR, fat, 0, arg->mtime, &subEntry.dir);
 	dir_write(&subEntry);
 
+	FLUSH((Stream_t *) Target);
 	subEntry.entry = 0;
 	GET_DATA(Target, 0, 0, 0, &fat);
 	mk_entry(".          ", ATTR_DIR, fat, 0, arg->mtime, &subEntry.dir);
@@ -101,7 +102,7 @@ static void usage(void)
 }
 
 Stream_t *createDir(Stream_t *Dir, const char *filename, ClashHandling_t *ch, 
-		    unsigned char attr, time_t mtime)
+					unsigned char attr, time_t mtime)
 {
 	CreateArg_t arg;
 	int ret;
@@ -110,7 +111,7 @@ Stream_t *createDir(Stream_t *Dir, const char *filename, ClashHandling_t *ch,
 	arg.attr = attr;
 	arg.mtime = mtime;
 
-	if (!getfreeMin(Dir, 1))
+	if (!getfreeMinClusters(Dir, 1))
 		return NULL;
 
 	ret = mwrite_one(Dir, filename,0, makeit, &arg, ch);
