@@ -4,7 +4,7 @@
  * written by:
  *
  * Alain L. Knaff			
- * Alain.Knaff@poboxes.com
+ * alain@linux.lu
  *
  */
 
@@ -13,7 +13,6 @@
 #include "mtools.h"
 
 #ifdef OS_linux
-#include "patchlevel.h"
 #include <sys/wait.h>
 #include "mainloop.h"
 #include "fs.h"
@@ -36,7 +35,7 @@ void mmount(int argc, char **argv, int type)
 		exit(1);
 	}
 	drive = toupper(argv[1][0]);
-	Stream = find_device(drive, O_RDONLY, &dev, &boot, name, &media);
+	Stream = find_device(drive, O_RDONLY, &dev, &boot, name, &media, 0);
 	if(!Stream)
 		exit(1);
 	FREE(&Stream);
