@@ -49,7 +49,7 @@ int renameit(char *dosname,
 	strncpy(targetEntry->dir.name, dosname, 8);
 	strncpy(targetEntry->dir.ext, dosname + 8, 3);
 
-	if(targetEntry->dir.attr & 0x10) {
+	if(IS_DIR(targetEntry)) {
 		direntry_t *movedEntry;
 
 		/* get old direntry. It is important that we do this
@@ -117,6 +117,7 @@ static int rename_file(direntry_t *entry, MainParam_t *mp)
 	if (targetDir == entry->Dir){
 		arg->ch.ignore_entry = -1;
 		arg->ch.source = entry->entry;
+		arg->ch.source_entry = entry->entry;
 	} else {
 		arg->ch.ignore_entry = -1;
 		arg->ch.source = -2;
@@ -172,6 +173,7 @@ static int rename_oldsyntax(direntry_t *entry, MainParam_t *mp)
 
 	arg->ch.ignore_entry = -1;
 	arg->ch.source = entry->entry;
+	arg->ch.source_entry = entry->entry;
 
 	if(!strcasecmp(mp->shortname, arg->fromname)){
 		longname = mp->longname;
@@ -268,7 +270,7 @@ void mmove(int argc, char **argv, int oldsyntax)
 		oldsyntax = 0;
 
 	arg.mp.lookupflags = 
-	  ACCEPT_PLAIN | ACCEPT_DIR | DO_OPEN_DIRS | NO_DOTS;
+	  ACCEPT_PLAIN | ACCEPT_DIR | DO_OPEN_DIRS | NO_DOTS | NO_UNIX;
 
 	if (!oldsyntax){
 		target_lookup(&arg.mp, argv[argc-1]);
