@@ -1,6 +1,7 @@
 /*
  * mformat.c
  */
+#define DONT_NEED_WAIT
 
 #include "sysincludes.h"
 #include "msdos.h"
@@ -14,6 +15,11 @@
 #include "scsi.h"
 #include "partition.h"
 
+#ifdef OS_linux
+#include "linux/hdreg.h"
+#include "linux/fs.h"
+#endif
+
 #define tolinear(x) \
 (sector(x)-1+(head(x)+cyl(x)*used_dev->heads)*used_dev->sectors)
 
@@ -238,7 +244,8 @@ static void usage(void)
 	fprintf(stderr, 
 		"Mtools version %s, dated %s\n", mversion, mdate);
 	fprintf(stderr, 
-		"Usage: %s [-prIadcv] [-s sectors] [-t cylinders] "
+		"Usage: %s [-pradcv] [-I [-B bootsect-template] [-s sectors] "
+			"[-t cylinders] "
 		"[-h heads] [-T type] [-b begin] [-l length] "
 		"drive\n", progname);
 	exit(1);
@@ -283,14 +290,18 @@ void mpartition(int argc, char **argv, int dummy)
 	struct partition *partTable=(struct partition *)(buf+ 0x1ae);
 	struct device *dev;
 	char errmsg[200];
- 
+	char *bootSector=0;
+
 	argtracks = 0;
 	argheads = 0;
 	argsectors = 0;
 
 	/* get command line options */
-	while ((c = getopt(argc, argv, "adprcIT:t:h:s:fvpb:l:S:")) != EOF) {
+	while ((c = getopt(argc, argv, "adprcIT:t:h:s:fvpb:l:S:B:")) != EOF) {
 		switch (c) {
+			case 'B':
+				bootSector = optarg;
+				break;
 			case 'a':
 				/* no privs, as it could be abused to
 				 * make other partitions unbootable, or
@@ -401,7 +412,7 @@ void mpartition(int argc, char **argv, int dummy)
 		expand(dev->name, name);
 		Stream = SimpleFileOpen(&used_dev, dev, name,
 					dirty ? O_RDWR : O_RDONLY, 
-					errmsg, open2flags, 1);
+					errmsg, open2flags, 1, 0);
 
 		if (!Stream) {
 			sprintf(errmsg,"init: open: %s", strerror(errno));
@@ -419,7 +430,7 @@ void mpartition(int argc, char **argv, int dummy)
 			memset ((void *) &cmd[2], 0, 8);
 			memset ((void *) &data[0], 137, 10);
 			scsi_cmd(get_fd(Stream), cmd, 10, SCSI_IO_READ,
-				 data, 10);
+				 data, 10, get_extra_data(Stream));
 			
 			tot_sectors = 1 +
 				(data[0] << 24) +
@@ -430,6 +441,12 @@ void mpartition(int argc, char **argv, int dummy)
 				printf("%d sectors in total\n", tot_sectors);
 		}
 
+#ifdef OS_linux
+		if (tot_sectors == 0) {
+			ioctl(get_fd(Stream), BLKGETSIZE, &tot_sectors);
+		}
+#endif
+
 		/* read the partition table */
 		if (READS(Stream, (char *) buf, 0, 512) != 512) {
 			sprintf(errmsg, 
@@ -457,8 +474,17 @@ void mpartition(int argc, char **argv, int dummy)
 	}
 
 	if(initialize) {
+		if (bootSector) {
+			int fd;
+			fd = open(bootSector, O_RDONLY);
+			if (fd < 0) {
+				perror("open boot sector");
+				exit(1);
+			}
+			read(fd, (char *) buf, 512);
+		}
 		memset((char *)(partTable+1), 0, 4*sizeof(*partTable));
-		set_dword(((char*)buf)+510, 0xaa55);
+		set_dword(((unsigned char*)buf)+510, 0xaa55);
 	}
 
 	/* check for boot signature, and place it if needed */
@@ -515,20 +541,32 @@ void mpartition(int argc, char **argv, int dummy)
 			end_set = 1;
 		}
 	}
-	
+
+#ifdef OS_linux
+	if(!used_dev.sectors && !used_dev.heads) {
+		if(!IS_SCSI(dev)) {
+			struct hd_geometry geom;
+			if(ioctl(get_fd(Stream), HDIO_GETGEO, &geom) == 0) {
+				used_dev.heads = geom.heads;
+				used_dev.sectors = geom.sectors;
+			}
+		}
+	}
+#endif
+
 	if(!used_dev.sectors && !used_dev.heads) {
 		if(tot_sectors)
 			setsize0(tot_sectors,&dummy2,&used_dev.heads,
-				 &used_dev.sectors);
+					 &used_dev.sectors);
 		else {
 			used_dev.heads = 64;
-			used_dev.heads = 32;
+			used_dev.sectors = 32;
 		}
 	}
 
 	if(verbose)
-		fprintf(stderr,"sectors: %d heads: %d\n",
-			used_dev.sectors, used_dev.heads);
+		fprintf(stderr,"sectors: %d heads: %d %d\n",
+			used_dev.sectors, used_dev.heads, tot_sectors);
 
 	sec_per_cyl = used_dev.sectors * used_dev.heads;
 	if(create) {
