@@ -67,9 +67,9 @@ UNUSED(static inline void set_word(unsigned char *data, unsigned short value))
 /*
  *	    hi byte     |    low byte
  *	|7|6|5|4|3|2|1|0|7|6|5|4|3|2|1|0|
- *      | | | | | | | | | | | | | | | | |
- *      \   7 bits    /\4 bits/\ 5 bits /
- *         year +80      month     day
+ *  | | | | | | | | | | | | | | | | |
+ *  \   7 bits    /\4 bits/\ 5 bits /
+ *     year +80     month     day
  */
 #define	DOS_YEAR(dir) (((dir)->date[1] >> 1) + 1980)
 #define	DOS_MONTH(dir) (((((dir)->date[1]&0x1) << 3) + ((dir)->date[0] >> 5)))
@@ -217,6 +217,8 @@ extern struct OldDos_t {
 extern const char *mversion;
 extern const char *mdate;
 
+extern char *Version;
+extern char *Date;
 
 
 int init(char drive, int mode);
