@@ -1,12 +1,9 @@
 #include "sysincludes.h"
 #include "msdos.h"
 #include "mtools.h"
-#include "patchlevel.h"
 #include "partition.h"
 #include "vfat.h"
 
-const char *mversion = VERSION;
-const char *mdate = DATE;
 const char *progname;
 
 static const struct dispatch {
@@ -16,11 +13,13 @@ static const struct dispatch {
 } dispatch[] = {
 	{"mattrib",mattrib, 0},
 	{"mbadblocks",mbadblocks, 0},
+	{"mcat",mcat, 0},
 	{"mcd",mcd, 0},
 	{"mcopy",mcopy, 0},
 	{"mdel",mdel, 0},
 	{"mdeltree",mdel, 2},
 	{"mdir",mdir, 0},
+	{"mdoctorfat",mdoctorfat, 0},
 	{"mdu",mdu, 0},
 	{"mformat",mformat, 0},
 	{"minfo", minfo, 0},
@@ -46,6 +45,9 @@ int main(int argc,char **argv)
 	int i;
 
 	init_privs();
+#ifdef __EMX__
+	_wildcard(&argc,&argv);
+#endif
 
 /*#define PRIV_TEST*/
 
@@ -79,38 +81,6 @@ int main(int argc,char **argv)
        _wildcard(&argc,&argv);
 #endif 
 
-	/* print the version */
-	if(argc >= 2 && strcmp(argv[1], "-V") == 0) {
-		printf("Mtools version %s, dated %s\n", mversion, mdate);
-		printf("configured with the following options: ");
-#ifdef USE_XDF
-		printf("enable-xdf ");
-#else
-		printf("disable-xdf ");
-#endif
-#ifdef USING_VOLD
-		printf("enable-vold ");
-#else
-		printf("disable-vold ");
-#endif
-#ifdef USING_NEW_VOLD
-		printf("enable-new-vold ");
-#else
-		printf("disable-new-vold ");
-#endif
-#ifdef DEBUG
-		printf("enable-debug ");
-#else
-		printf("disable-debug ");
-#endif
-#ifdef USE_RAWTERM
-		printf("enable-raw-term ");
-#else
-		printf("disable-raw-term ");
-#endif
-		printf("\n");
-		return 0;
-	}
 
 	/* check whether the compiler lays out structures in a sane way */
 	if(sizeof(struct partition) != 16 ||
@@ -141,6 +111,44 @@ int main(int argc,char **argv)
 		name = argv[0];
 	}
 
+
+
+	/* print the version */
+	if(argc >= 2 && 
+	   (strcmp(argv[1], "-V") == 0 || strcmp(argv[1], "--version") ==0)) {
+		printf("%c%s version %s, dated %s\n", 
+		       toupper(name[0]), name+1,
+		       mversion, mdate);
+		printf("configured with the following options: ");
+#ifdef USE_XDF
+		printf("enable-xdf ");
+#else
+		printf("disable-xdf ");
+#endif
+#ifdef USING_VOLD
+		printf("enable-vold ");
+#else
+		printf("disable-vold ");
+#endif
+#ifdef USING_NEW_VOLD
+		printf("enable-new-vold ");
+#else
+		printf("disable-new-vold ");
+#endif
+#ifdef DEBUG
+		printf("enable-debug ");
+#else
+		printf("disable-debug ");
+#endif
+#ifdef USE_RAWTERM
+		printf("enable-raw-term ");
+#else
+		printf("disable-raw-term ");
+#endif
+		printf("\n");
+		return 0;
+	}
+
 	read_config();
 	setup_signal();
 	for (i = 0; i < NDISPATCH; i++) {
