@@ -15,6 +15,8 @@ extern int lockf(int, int, off_t);  /* SCO has no proper include file for lockf
 #define USE_XDF_FLAG 8
 #define MFORMAT_ONLY_FLAG 16
 #define VOLD_FLAG 32
+#define FLOPPYD_FLAG 64
+#define FILTER_FLAG 128
 
 #define IS_SCSI(x)  ((x) && ((x)->misc_flags & SCSI_FLAG))
 #define IS_PRIVILEGED(x) ((x) && ((x)->misc_flags & PRIV_FLAG))
@@ -29,18 +31,18 @@ typedef struct device {
 	char drive;	   	    	/* the drive letter */
 	int fat_bits;			/* FAT encoding scheme */
 
-	int mode;		       	/* any special open() flags */
+	unsigned int mode;		/* any special open() flags */
 	unsigned int tracks;	/* tracks */
 	unsigned int heads;		/* heads */
 	unsigned int sectors;	/* sectors */
-	int hidden;				/* number of hidden sectors. Used for
+	unsigned int hidden;	/* number of hidden sectors. Used for
 							 * mformatting partitioned devices */
 
 	off_t offset;	       	/* skip this many bytes */
 
-	int partition;
+	unsigned int partition;
 
-	int misc_flags;
+	unsigned int misc_flags;
 
 	/* Linux only stuff */
 	unsigned int ssize;
@@ -52,6 +54,8 @@ typedef struct device {
 	/* internal variables */
 	int file_nr;		/* used during parsing */
 	int blocksize;	        /* size of disk block in bytes */
+
+	const char *cfg_filename; /* used for debugging purposes */
 } device_t;
 
 
@@ -98,11 +102,10 @@ int lock_dev(int fd, int mode, struct device *dev);
 char *unix_normalize (char *ans, char *name, char *ext);
 char *dos_name(char *filename, int verbose, int *mangled, char *buffer);
 struct directory *mk_entry(const char *filename, char attr,
-			   unsigned int fat, size_t size, long date,
+			   unsigned int fat, size_t size, time_t date,
 			   struct directory *ndir);
 int copyfile(Stream_t *Source, Stream_t *Target);
-unsigned long getfree(Stream_t *Stream);
-unsigned long getfreeMin(Stream_t *Stream, size_t ref);
+int getfreeMinClusters(Stream_t *Stream, size_t ref);
 
 FILE *opentty(int mode);
 
@@ -112,9 +115,6 @@ void bufferize(Stream_t **Dir);
 int dir_grow(Stream_t *Dir, int size);
 int match(const char *, const char *, char *, int, int);
 
-Stream_t *new_file(char *filename, char attr,
-		   unsigned int fat, size_t size, long date,
-		   struct directory *ndir);
 char *unix_name(char *name, char *ext, char Case, char *answer);
 void *safe_malloc(size_t size);
 Stream_t *open_filter(Stream_t *Next);
@@ -136,23 +136,21 @@ UNUSED(static inline int compare (long ref, long testee))
 }
 
 Stream_t *GetFs(Stream_t *Fs);
-Stream_t *find_device(char drive, int mode, struct device *out_dev,
-		      struct bootsector *boot,
-		      char *name, int *media);
 
 char *label_name(char *filename, int verbose, 
 		 int *mangled, char *ans);
 
 /* environmental variables */
-extern int mtools_skip_check;
-extern int mtools_fat_compatibility;
-extern int mtools_ignore_short_case;
-extern int mtools_no_vfat;
-extern int mtools_numeric_tail;
-extern int mtools_dotted_dir;
-extern int mtools_twenty_four_hour_clock;
+extern unsigned int mtools_skip_check;
+extern unsigned int mtools_fat_compatibility;
+extern unsigned int mtools_ignore_short_case;
+extern unsigned int mtools_no_vfat;
+extern unsigned int mtools_numeric_tail;
+extern unsigned int mtools_dotted_dir;
+extern unsigned int mtools_twenty_four_hour_clock;
 extern char *mtools_date_string;
-extern int mtools_rate_0, mtools_rate_any, mtools_raw_tty;
+extern unsigned int mtools_rate_0, mtools_rate_any;
+extern int mtools_raw_tty;
 
 extern int batchmode;
 
@@ -168,10 +166,12 @@ extern const int nr_const_devices;
 
 void mattrib(int argc, char **argv, int type);
 void mbadblocks(int argc, char **argv, int type);
+void mcat(int argc, char **argv, int type);
 void mcd(int argc, char **argv, int type);
 void mcopy(int argc, char **argv, int type);
 void mdel(int argc, char **argv, int type);
 void mdir(int argc, char **argv, int type);
+void mdoctorfat(int argc, char **argv, int type);
 void mdu(int argc, char **argv, int type);
 void mformat(int argc, char **argv, int type);
 void minfo(int argc, char **argv, int type);
@@ -184,6 +184,7 @@ void mshowfat(int argc, char **argv, int mtype);
 void mtoolstest(int argc, char **argv, int type);
 void mzip(int argc, char **argv, int type);
 
+extern int noPrivileges;
 void init_privs(void);
 void reclaim_privs(void);
 void drop_privs(void);
@@ -211,7 +212,6 @@ struct dirCache_t **getDirCacheP(Stream_t *Stream);
 int isRootDir(Stream_t *Stream);
 unsigned int getStart(Stream_t *Dir, struct directory *dir);
 unsigned int countBlocks(Stream_t *Dir, unsigned int block);
-size_t countBytes(Stream_t *Dir, unsigned int block);
 char getDrive(Stream_t *Stream);
 
 
@@ -222,7 +222,9 @@ char *get_homedir(void);
 const char *expand(const char *, char *);
 const char *fix_mcwd(char *);
 FILE *open_mcwd(const char *mode);
-void unlink_mcwd();
+void unlink_mcwd(void);
+
+int safePopenOut(char **command, char *output, int len);
 
 #define ROUND_DOWN(value, grain) ((value) - (value) % (grain))
 #define ROUND_UP(value, grain) ROUND_DOWN((value) + (grain)-1, (grain))
