@@ -134,7 +134,7 @@ void clear_vfat(struct vfat_state *v)
  * Thanks to Jeffrey Richter of Microsoft Systems Journal for
  * pointing me to the correct algorithm.
  *
- * David C. Niemi (niemidc@erols.com) 95.01.19
+ * David C. Niemi (niemi@tux.org) 95.01.19
  */
 static inline unsigned char sum_shortname(char *name)
 {
@@ -197,7 +197,7 @@ void clear_vses(Stream_t *Dir, int entrySlot, size_t last)
 	addFreeEntry(cache, entry.entry, last);
 	for (; entry.entry < last; ++entry.entry) {
 #ifdef DEBUG
-		fprintf(stderr,"Clearing entry %d.\n", entry);
+		fprintf(stderr,"Clearing entry %d.\n", entry.entry);
 #endif
 		dir_read(&entry);
 		if(!entry.dir.name[0] || entry.dir.name[0] == DELMARK)
@@ -232,7 +232,7 @@ int write_vfat(Stream_t *Dir, char *shortname, char *longname, int start,
 		vse->sum = sum_shortname(shortname);
 #ifdef DEBUG
 		printf("Wrote checksum=%d for shortname %s.\n", 
-		       vse.sum,shortname);
+		       vse->sum,shortname);
 #endif
 		num_vses = strlen(longname)/VSE_NAMELEN + 1;
 		for (vse_id = num_vses; vse_id; --vse_id) {
@@ -273,6 +273,11 @@ void dir_write(direntry_t *entry)
 	dirCacheEntry_t *dce;
 	dirCache_t *cache;
 
+	if(entry->entry == -3) {
+		fprintf(stderr, "Attempt to write root directory pointer\n");
+		exit(1);
+	}
+
 	cache = allocDirCache(entry->Dir, entry->entry + 1);
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in dir_write\n");
@@ -492,11 +497,6 @@ static result_t checkNameForMatch(struct direntry_t *direntry,
 
 
 	/*---------- multiple files ----------*/
-#ifdef DEBUG
-	printf(
-		"!single, vfat_present=%d, vfat.name=%s, newfile=%s, filename=%s.\n",
-		vfat->present, vfat->name, newfile, filename);
-#endif						
 	if(!((flags & MATCH_ANY) ||
 	     (dce->longName && 
 	      match(dce->longName, filename, direntry->name, 0, length)) ||
@@ -537,7 +537,6 @@ static result_t checkNameForMatch(struct direntry_t *direntry,
 int vfat_lookup(direntry_t *direntry, const char *filename, int length,
 		int flags, char *shortname, char *longname)
 {
-	/*size_t len;*/
 	dirCacheEntry_t *dce;
 	result_t result;
 	dirCache_t *cache;
@@ -548,7 +547,6 @@ int vfat_lookup(direntry_t *direntry, const char *filename, int length,
 	if (direntry->entry == -2)
 		return -1;
 
-	/*GET_DATA(direntry->Dir, 0, &len, 0, 0);*/	
 	cache = allocDirCache(direntry->Dir, direntry->entry+1);
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in vfat_lookup [0]\n");
